-------------------------------------------------------------------------
-- Joseph Zambreno
-- Department of Electrical and Computer Engineering
-- Iowa State University
-------------------------------------------------------------------------


-- Quadratic.vhd
-------------------------------------------------------------------------
-- DESCRIPTION: This file contains an implementation of the Quadratic
-- equation Ax^2+Bx+C using invidual adder and multiplier units.
--
--
-- NOTES:
-- 9/7/06 by JAZ::Design created.
-------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity Quadratic is

  generic(gDATA_WIDTH   : natural := 8);
  port(iCLK             : in std_logic;
       iX 		: in std_logic_vector(gDATA_WIDTH-1 downto 0);
       oY 		: out std_logic_vector(2*gDATA_WIDTH+8-1 downto 0));

end Quadratic;

architecture dataflow of Quadratic is
  component Adder
    generic(gDATA_WIDTH1 : natural := 8;
	    gDATA_WIDTH2 : natural := 8);
    port(iCLK           : in std_logic;
         iA             : in std_logic_vector(gDATA_WIDTH1-1 downto 0);
         iB             : in std_logic_vector(gDATA_WIDTH2-1 downto 0);
         oC             : out std_logic_vector(gDATA_WIDTH1-1 downto 0));
  end component;

  component Multiplier
    generic(gDATA_WIDTH1 : natural := 8;
	    gDATA_WIDTH2 : natural := 8);
    port(iCLK           : in std_logic;
         iA             : in std_logic_vector(gDATA_WIDTH1-1 downto 0);
         iB             : in std_logic_vector(gDATA_WIDTH2-1 downto 0);
         oC             : out std_logic_vector(gDATA_WIDTH1+gDATA_WIDTH2-1 downto 0));
  end component;

  -- Constants for the A, B, C values. No need to change these.
  constant cA : std_logic_vector(7 downto 0) := "01100110";
  constant cB : std_logic_vector(7 downto 0) := "00001111";
  constant cC : std_logic_vector(7 downto 0) := "01010101";

  -- Signals to store A*x, B*x
  signal sVALUE_Ax, sVALUE_Bx : std_logic_vector(gDATA_WIDTH+8-1 downto 0);
  -- Signal to store A*x*x
  signal sVALUE_Axx 	      : std_logic_vector(2*gDATA_WIDTH+8-1 downto 0);
  -- Signal to store B*x+C
  signal sVALUE_BxpC          : std_logic_vector(gDATA_WIDTH+8-1 downto 0);

begin

  
  ---------------------------------------------------------------------------
  -- Level 1: Compute A*x, B*x
  ---------------------------------------------------------------------------
  g_Mult1: Multiplier
    generic map(gDATA_WIDTH1  => gDATA_WIDTH,
		        gDATA_WIDTH2  => 8)
    port MAP(iCLK             => iCLK,
             iA               => ,
             iB               => ,
             oC               => );

  g_Mult2: Multiplier
    generic map(gDATA_WIDTH1  => gDATA_WIDTH,
		        gDATA_WIDTH2  => 8)
    port MAP(iCLK             => iCLK,
             iA               => ,
             iB               => ,
             oC               => );
    
 ---------------------------------------------------------------------------
  -- Level 2: Calculate A*x*x, B*x+C
  ---------------------------------------------------------------------------
  g_Mult3: Multiplier
    generic map(gDATA_WIDTH1  => gDATA_WIDTH+8,
		        gDATA_WIDTH2  => gDATA_WIDTH)
    port MAP(iCLK             => iCLK,
             iA               => ,
             iB               => ,
             oC               => );

  g_Add1: Adder
    generic map(gDATA_WIDTH1  => gDATA_WIDTH+8,
		        gDATA_WIDTH2  => 8)
    port MAP(iCLK             => iCLK,
             iA               => ,
             iB               => ,
             oC               => );
    
  ---------------------------------------------------------------------------
  -- Level 3: Calculate A*x*x + B*x + C
  ---------------------------------------------------------------------------
  g_Add2: Adder
    generic map(gDATA_WIDTH1  => 2*gDATA_WIDTH+8,
		        gDATA_WIDTH2  => gDATA_WIDTH+8)
    port MAP(iCLK             => iCLK,
             iA               => ,
             iB               => ,
             oC               => );
  
end dataflow;
